from dal import autocomplete
from django.db.models import Q
from django.db import models

from dj_tickets.core.models import SupportTeam, SupportTeamMember
from django.contrib.auth import get_user_model
from django.template.loader import render_to_string


User = get_user_model()

# Get all character fields from the User model
char_fields = [
    field.name
    for field in User._meta.get_fields()
    if isinstance(field, (models.CharField, models.TextField,models.EmailField, models.IntegerField, models.URLField))
    and field.name != 'password'
]



class SupportTeamAutocomplete(autocomplete.Select2QuerySetView):
    
    def get_result_label(self, result):
        return result.name

    def get_queryset(self):
        # Don't forget to filter out results depending on the visitor !
        if not self.request.user.is_authenticated:
            return SupportTeam.objects.none()

        qs = SupportTeam.objects.all()

        if self.q:
            qs = qs.filter(name__icontains=self.q)

        return qs
    

class SupportTeamMemberAutocomplete(autocomplete.Select2QuerySetView):
    
    template = 'dj_tickets/mtv/autocompletes/support_team_member_autocomplete.html'

    def get_selected_result_label(self, result):
        return str(result.user)

    
    def get_queryset(self):
        # Don't forget to filter out results depending on the visitor !
        if not self.request.user.is_authenticated:
            return SupportTeamMember.objects.none()

        qs = SupportTeamMember.objects.all()

        # Build a Q object for filtering
        if self.q:

            search_query = Q()

            for field in char_fields:

                search_query |= Q(**{f"user__{field}__icontains": self.q})
            
            qs = qs.filter(search_query)
        
        qs = qs.filter(support_team_id=self.request.GET.get('support_team',-1))

        return qs
    

class UserAutocomplete(autocomplete.Select2QuerySetView):
    
    template = 'dj_tickets/mtv/autocompletes/user_autocomplete.html'

    def get_selected_result_label(self, result):
        return str(result)

    def get_queryset(self):
        # Don't forget to filter out results depending on the visitor !
        if not self.request.user.is_authenticated:
            return User.objects.none()

        qs = User.objects.all()

        # Build a Q object for filtering

        if self.q:
            search_query = Q()
            for field in char_fields:
                search_query |= Q(**{f"{field}__icontains": self.q})
            
            qs = qs.filter(search_query)

        return qs