from typing import Any, Mapping
from django import forms
from django.core.files.base import File
from django.db.models.base import Model
from django.forms.utils import ErrorList
from dj_tickets.core.models import Ticket,SupportTeam,DefaultTicket
from dal import autocomplete
from django.contrib.auth import get_user_model

User = get_user_model()

class TicketCreateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = [
            'content',
            'observers',
            'support_team',
        ]
        widgets = {
            'support_team': autocomplete.ModelSelect2(url='dj_tickets:support-team-autocomplete'),
            'observers': autocomplete.ModelSelect2Multiple(url='dj_tickets:user-autocomplete',attrs={
                    'data-html':True
                }),
        }



class AdminTicketUpdateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = [
            'status',
            'observers',
            'support_team',
            'support_team_member',
        ]
        widgets = {
            'support_team': autocomplete.ModelSelect2(url='dj_tickets:support-team-autocomplete'),
            'observers': autocomplete.ModelSelect2Multiple(url='dj_tickets:user-autocomplete',attrs={
                    'data-html':True
                }),
            'support_team_member': autocomplete.ModelSelect2(url='dj_tickets:support-team-member-autocomplete',attrs={
                    'data-html':True
                }),
        }


class ClientAdminTicketUpdateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = [
            'status',
            'observers',
            'support_team',
            'support_team_member',
            'is_resolved',
        ]
        widgets = {
            'support_team': autocomplete.ModelSelect2(url='dj_tickets:support-team-autocomplete'),
            'observers': autocomplete.ModelSelect2Multiple(url='dj_tickets:user-autocomplete',attrs={
                    'data-html':True
                }),
            'support_team_member': autocomplete.ModelSelect2(url='dj_tickets:support-team-member-autocomplete',attrs={
                    'data-html':True
                }),
        }


class ClientTicketUpdateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = [
            'observers',
            'is_resolved',
        ]
        widgets = {
            'observers': autocomplete.ModelSelect2Multiple(url='dj_tickets:user-autocomplete',attrs={
                    'data-html':True
                }),
        }
    
    # def __init__(self, *args, **kwargs) -> None:
    #     super().__init__()
    #     self.fields['observers'].queryset = User.objects.none()

    def save(self, commit: bool = ...) -> Any:
        observers = self.cleaned_data.pop('observers',User.objects.none())
        self.instance : DefaultTicket
        self.instance.observers.add(*observers)
        return super().save(commit)


class SupportMemberTicketUpdateForm(forms.ModelForm):
    status = forms.ChoiceField(choices=[item for item in Ticket.Status.choices if item[0] !='pending'])
    class Meta:
        model = Ticket
        fields = [
            'status',
        ]
    
class ClientSupportMemberTicketUpdateForm(forms.ModelForm):
    class Meta:
        model = Ticket
        fields = [
            'status',
            'observers',
            'support_team',
            'is_resolved',
        ]
        widgets = {
            'support_team': autocomplete.ModelSelect2(url='dj_tickets:support-team-autocomplete'),
            'observers': autocomplete.ModelSelect2Multiple(url='dj_tickets:user-autocomplete',attrs={
                    'data-html':True
                }),
        }